#include "utils.h"

/*
 * Create  an agent that waits on a specified event on document and
 * then sends mail with the specified content.
 * It  also does a HTTP POST to  a specified URL with the contents
 * specified. This can be used to trigger remote CGI programs.
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * documentName - the URI of the document on which to wait
 * event - the event number to watch (see list above)
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when 
							agent is created
 * mailAndNewsmessage - the message to be sent in the email 
							when activated
 * newsgroup - name of newsgroup
 * posturl - the url to POST to when activated
 * postmsg - message to POST
 *
 * WARNING: 
   This agent harcodes the username and password (user, user), and
   sends it uuencoded over the wire. We do not recommend talking to
   servers requiring authorization using the agent POST services.
   This service should be used to talk to CGI and other servers that
   do not require authorization. The example here demonstrates what
    can be done and it encodes the entire message (including the name
	and password) into the message. The agent server just POSTs the 
	message as is. If you are comfortable with sending passwords over
	the wire without secure encodng, by all means go ahead.
 */

int CreatePOSTMailAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		const char* documentName, 
		int   event,
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* mailAndNewsmessage,
		const char* newsgroup, /* can be 0 */
		const char* posturl,
		const char* postmsg,
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";
	char 		buf[1024];
	
	EncodeUrl(buf, postmsg); /* takes care of special characters */


	EncodeUrl(document, documentName);

#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Document: %s\n", document);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("Event: %d\n", event);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
	printf("POST url: %s\n", posturl);
	printf("Agent Name: %s\n", agentName);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=DOCUMENT&\
nsDocIsDir=0&\
actionType=e-mail&\
timerType=0&\
agentName=%s&\
documentURL=%s&\
documentEventType=%d&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s&\
postURL=%s&\
postMsg=%s",
			agentName, document, event, actionEmailAddr, mailAndNewsmessage, 
				(newsgroup?newsgroup:""), notifyEmailAddr, posturl, buf);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


/*
 * Create  an agent that waits on a specified event on document and
 * then sends mail with the specified content.
 * It  also does a HTTP GET from a specified URL and
 * appends the contents of the document to the mail meesage 
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * documentName - the URI of the document on which to wait
 * event - the event number to watch (see list above)
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * mailAndNewsmessage - the message to be sent in the email when activated
 * newsgroup - name of newsgroup
 * geturl - the url to get when activated
 */

int CreateGETAndMailAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		const char* documentName, 
		int   event,
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* mailAndNewsmessage,
		const char* newsgroup, /* can be 0 */
		const char* geturl,
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";

	EncodeUrl(document, documentName);

#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Document: %s\n", document);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("Event: %d\n", event);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
	printf("GET url: %s\n", geturl);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=DOCUMENT&\
nsDocIsDir=0&\
actionType=e-mail&\
timerType=0&\
agentName=%s&documentURL=%s&\
documentEventType=%d&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s&\
getURL=%s",
			agentName, document, event, actionEmailAddr, mailAndNewsmessage, 
				(newsgroup?newsgroup:""), notifyEmailAddr, geturl);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}



/*
 * Create  a search agent that waits on a timer and
 * then searches a collection ends mail to the address specified with the 
 * results of the search
 * it is very similar to a timer agent
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * message - the message to be sent in the email when activated

 * daysvector:  Activation days of the timer 
                Monday is 0 and Tuesday is 1... Sunday is 7
                daysVector[0] = 1 (activate on Monday)
                daysVector[1] = 1 (activate on Tuesday)
                ...
                daysVector[6] = 0 (do not activate on Sunday)
        If the corresponding entry is not zero, the timer is activated for that
        day.
 * start - time in seconds (UTC time) when the first alarm will occur. 
 * end     when this alarm will expire
 * collectionName - Name of the collection to search when activated
 * searchPattern  - pattern to look for in the documents selected
 * searchCriteria - enum value, search in all documents or updated documents
 */

int CreateWeeklySearchAndMailAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		short dayvector[], /* dayvector[7] */
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* message,
		const char* newsgroup,
		time_t start, /* time in UTC */
		time_t end, /* time in UTC */
		const char* collectionName,
		const char* searchPattern,
		ESearchType searchCriteria,
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";
	char		days[8];
	int			i;

	for(i=0; i < 7; i++)
		if (dayvector[i])
			days[i] = 'Y';
	else days[i] ='N';
	days[7] = 0;


#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Activated days : %s\n", days);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=TIMER&\
isSearchAgent=1&\
actionType=e-mail&\
dayVector=%s&\
timerType=%d&\
startTime=%ld&\
endTime=%ld&\
agentName=%s&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s&\
doSearch=1&\
searchType=%d&\
collectionName=%s&\
searchPattern=%s",
			days, (int)eTimerSelective, start, end, 
			 agentName, actionEmailAddr, message, (newsgroup?newsgroup:""),
			notifyEmailAddr, 
			(int)searchCriteria, collectionName, searchPattern);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


/*
 * Create  a search agent that waits on a timer and
 * then searches a collection ends mail to the address specified with the 
 * results of the search
 * it is very similar to a timer agent
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * message - the message to be sent in the email when activated
 * period - the frequency, depends on the timertype. If eTimerPeriodicMinutes
            this is minutes between triggers. If eTimerPeriodicHours, this is
            the hours between triggers, and so on.
 * timertype - One of 
        eTimerPeriodicMinutes
        eTimerPeriodicHours
        eTimerPeriodicDays
        eTimerPeriodicWeeks
        eTimerPeriodicMonths
        eTimerPeriodicYears

 * start - time in seconds (UTC time) when the first alarm will occur. 
			A call to time() return time_t
 * end     when this alarm will expire
 * collectionName - Name of the collection to search when activated
 * searchPattern  - pattern to look for in the documents selected
 * searchCriteria - enum value, search in all documents or updated documents
 */

int CreatePeriodicSearchAndMailAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		int period,
		ETimerType timertype, /* eTimerPeriodicMinutes, eTimerPeriodicHours, */
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* message,
		const char* newsgroup,
		time_t start, /* time in UTC */
		time_t end, /* time in UTC */
		const char* collectionName,
		const char* searchPattern,
		ESearchType searchCriteria,
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";

	switch (timertype) {
		case eTimerPeriodicMinutes:
		case eTimerPeriodicHours:
		case eTimerPeriodicDays:
		case eTimerPeriodicWeeks:
		case eTimerPeriodicMonths:
		case eTimerPeriodicYears:
			break;
		default:
			return FAILURE;
	}


#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Timer type: %d\n", timertype);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=TIMER&\
isSearchAgent=1&\
actionType=e-mail&\
timerType=%d&\
startTime=%d&\
endTime=%d&\
period=%d&\
periodUnit=%d&\
agentName=%s&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s&\
doSearch=1&\
searchType=%d&\
collectionName=%s&\
searchPattern=%s",
			(int)timertype, start, end, period, (int)timertype,
			 agentName, actionEmailAddr, message, (newsgroup?newsgroup:""),
			notifyEmailAddr, 
			(int)searchCriteria, collectionName, searchPattern);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}





/*
 * Create  an agent that is activated by a timer alarm.
 * When activated it sends mail to the address specified
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * message - the message to be sent in the email when activated
 * period - the frequency, depends on the timertype. If eTimerPeriodicMinutes
            this is minutes between triggers. If eTimerPeriodicHours, this is
            the hours between triggers, and so on.
 * timertype - One of 
        eTimerPeriodicMinutes
        eTimerPeriodicHours
        eTimerPeriodicDays
        eTimerPeriodicWeeks
        eTimerPeriodicMonths
        eTimerPeriodicYears

 * start - time in seconds (UTC time) when the first alarm will occur. 
			A call to time() return time_t
 * end     when this alarm will expire
 */

int CreateSpecificTimeMailAndNewsAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* message,
		const char* newsgroup,
		time_t when, /* When to notify, time in UTC */
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";


#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=TIMER&\
actionType=e-mail&\
timerType=%d&\
startTime=%d&\
agentName=%s&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s",
	(int)eTimerSpecific, when, agentName, actionEmailAddr, message, 
	(newsgroup?newsgroup:""), notifyEmailAddr);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


/*
 * Create  an agent that is activated by a timer alarm.
 * When activated it sends mail to the address specified
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * message - the message to be sent in the email when activated

 * daysvector:  Activation days of the timer 
                Monday is 0 and Tuesday is 1... Sunday is 7
                daysVector[0] = 1 (activate on Monday)
                daysVector[1] = 1 (activate on Tuesday)
                ...
                daysVector[6] = 0 (do not activate on Sunday)
        If the corresponding entry is not zero, the timer is activated for that
        day.
 * start - time in seconds (UTC time) when the first alarm will occur. 
 * end     when this alarm will expire
 */

int CreateWeeklyMailAndNewsAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		short dayvector[], /* dayvector[7] */
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* message,
		const char* newsgroup,
		time_t start, /* time in UTC */
		time_t end, /* time in UTC */
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";
	char		days[8];
	int			i;

	for(i=0; i < 7; i++)
		if (dayvector[i])
			days[i] = 'Y';
	else days[i] ='N';
	days[7] = 0;


#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Activated days : %s\n", days);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=TIMER&\
actionType=e-mail&\
dayVector=%s&\
timerType=%d&\
startTime=%ld&\
endTime=%ld&\
agentName=%s&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s",
			days, (int)eTimerSelective, start, end, 
			 agentName, actionEmailAddr, message, (newsgroup?newsgroup:""),
			notifyEmailAddr);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


/*
 * Create  an agent that is activated by a timer alarm.
 * When activated it sends mail to the address specified
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * message - the message to be sent in the email when activated
 * period - the frequency, depends on the timertype. If eTimerPeriodicMinutes
            this is minutes between triggers. If eTimerPeriodicHours, this is
            the hours between triggers, and so on.
 * timertype - One of 
        eTimerPeriodicMinutes
        eTimerPeriodicHours
        eTimerPeriodicDays
        eTimerPeriodicWeeks
        eTimerPeriodicMonths
        eTimerPeriodicYears

 * start - time in seconds (UTC time) when the first alarm will occur. 
			A call to time() return time_t
 * end     when this alarm will expire
 */

int CreatePeriodicMailAndNewsAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		int period,
		ETimerType timertype, /* eTimerPeriodicMinutes, eTimerPeriodicHours, */
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* message,
		const char* newsgroup,
		time_t start, /* time in UTC */
		time_t end, /* time in UTC */
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";

	switch (timertype) {
		case eTimerPeriodicMinutes:
		case eTimerPeriodicHours:
		case eTimerPeriodicDays:
		case eTimerPeriodicWeeks:
		case eTimerPeriodicMonths:
		case eTimerPeriodicYears:
			break;
		default:
			return FAILURE;
	}


#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Timer type: %d\n", timertype);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=TIMER&\
actionType=e-mail&\
timerType=%d&\
startTime=%d&\
endTime=%d&\
period=%d&\
periodUnit=%d&\
agentName=%s&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s",
			(int)timertype, start, end, period, (int)timertype,
			 agentName, actionEmailAddr, message, (newsgroup?newsgroup:""),
			notifyEmailAddr);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


/*
 * Create  an agent that waits on a specified event on document and
 * then sends mail to the address specified
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * documentName - the URI of the document on which to wait
 * event - the event number to watch (see list above)
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * mailAndNewsmessage - the message to be sent in the email when activated
 * newsgroup - name of newsgroup
 */

int CreateDocumentMailAndNewsAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		const char* documentName, 
		int   event,
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* mailAndNewsmessage,
		const char* newsgroup, /* can be 0 */
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";
	FILE *fp;

	EncodeUrl(document, documentName);

#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Document: %s\n", document);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("Event: %d\n", event);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=DOCUMENT&\
nsDocIsDir=0&\
actionType=e-mail&\
timerType=0&\
agentName=%s&documentURL=%s&\
documentEventType=%d&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s",
			agentName, document, event, actionEmailAddr, mailAndNewsmessage, 
				(newsgroup?newsgroup:""), notifyEmailAddr);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);

	fp = fopen("POST.dat","w");
	fprintf(fp, "%s", auth);
	fclose(fp);

	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


/*
 * Create  an agent that waits on a specified event on document and
 * then sends mail to the address specified
 *
 * server - the server name (sphinx.mcom.com, etc.)
 * port   - the port on which the server listens
 * agentName - what do you want to call this agent?
 * user  - name of the user who is creating the agent
 * passwd - the user's password, plain ascii 
         The (username, password) combination is used for authentication
 * documentName - the URI of the document on which to wait
 * event - the event number to watch (see list above)
 * actionEmailAddr - send mail to this address when activated
 * notificationEmailAddr - send mail to this address when agent is created
 * message - the message to be sent in the email when activated
 */

int CreateDirectoryMailAndNewsAgent(
		const char* server, 
		int port,
		const char* agentName,
		const char* user, 
		const char* passwd,
		const char*  directory, 
		int   event,
		const char* actionEmailAddr,
		const char* notifyEmailAddr,
		const char* message,
		const char* newsgroup, /* can be 0 */
		char* agentId   /* return value */)
{
	char* 		auth = 0;
	char       *response = NULL;
	char       *response_header = NULL;
	int 		status = 0;
    char 		header[1024];
    char 		body[1024];
    char 		post_msg[1024];
	char document[512];
	int 		rc = SUCCESS;
	const char* uri = "/Agents";

	EncodeUrl(document, directory);

#ifdef WIN32

    if (gSocketInitDone == 0) {
        wVersionRequested = MAKEWORD(1, 1);

        err = WSAStartup(wVersionRequested, &wsaData);
        gSocketInitDone = 1;

        if (err != 0) {
            /* Tell the user that we couldn't find a useable */
            /* winsock.dll.     */
            printf("Failed to initialize socket library\n");
            return (-1);
        }
    }
#endif

#ifdef DEBUG
	printf("server: %s\n", server);
	printf("Directory: %s\n", directory);
	printf("Email Address: %s\n", actionEmailAddr);
	printf("Event: %d\n", event);
	printf("User: %s\n", user);
	printf("Password: %s\n", passwd);
#endif

	MakeBasicAuthentication(user, passwd, &auth);

	sprintf(body, 
"NS-agent-page=create&\
NS-agent-action=yes&\
NS-agent-type=DOCUMENT&\
nsDocIsDir=1&\
actionType=e-mail&\
agentName=%s&\
documentURL=%s&\
documentEventType=%d&\
actionEmailAddr=%s&\
userMesg=%s&\
newsGroup=%s&\
notifyEmailAddr=%s",
			agentName, directory, event, actionEmailAddr, message, 
			(newsgroup?newsgroup:""), notifyEmailAddr);

	sprintf(header, "Content-Length:%d\n\
	Content-Type: application/x-www-form-urlencoded; charset=US-ASCII\n\
	Content-Transfer-Encoding: 7bit", strlen(body));

	/* build post message  */
	sprintf(post_msg, "%s\n\n%s\n", header, body);


	rc = DoHttpPost(server, port, uri, post_msg, auth,  &response_header, 
						&response, &status);


	if (response && strstr(response, "var result = \"Success\"")) {
		rc = SUCCESS;
		if (agentId != 0) 
			GetAgentId(response, agentId);
	}
	else rc = 0;
	
#ifdef DEBUG
	printf("Response status: %d\n", status);
	printf("Response header: %s\n", response_header);
	printf("Response body: %s\n", response);
#endif

	if (response_header)
		free(response_header);
	if (response) {
		free(response);
	}

	free(auth);

	return rc;
}


main(int argc, char* argv[])
{
	int rc = 0;

 
	const char* server = "hyperion.mcom.com";
	int port = 80;
	int event = NS_MODIFY_MSG;
	const char* user = "user";
	const char* password = "user";
	const char* document = "/index.html";
	const char* directory = "/testdir";
	const char* actionEmail = "user";
	const char* notificationEmail = "user";
	const char* agentName;
	const char* message = "Agent created from Remote Machine";
	const char* getThisUrl = "http://www.yahoo.com";
	const char* postToUrl = "http://hyperion/agents";
	char buf[56];
	char id[56];
	time_t start, end;
	int option = 0;
    /* Trigger Monday and Friday */
    short daysvector[] = {1, /* Mon */
                          0, /* Tue */
                          0, /* Wed */
                          0, /* Thu */
                          1, /* Fri */
                          0, /* Sat */
                          0  /* Sun */
                        };
	id[0] = 0;

	if (argc < 3) {
		printf("Usage: create testnumber [agentname]\n");
		exit(-1);
	}

	option = atoi(argv[1]);
	if (argc > 2) agentName = argv[2];
	else {
		time_t t = time(0);
		sprintf(buf, "AGENT_%ld", t);
		agentName = buf;
	}

	start = time(0)+2000;
	end = start+200000;


	if (option == 0) 
		rc = CreateDirectoryMailAndNewsAgent(server, 	/* server */
			port, 				/* port */
			agentName,
			user, 				/* user name */
			password, 			/* passwd */
			directory, 			/* const char* doc, */
			event, 				/* event */
			actionEmail, 		/* action email */
			notificationEmail,	/* notify email */
			message,          /* message */
			0,
			id   /* out value */
		);

	else if (option==1)
		rc = 	CreatePeriodicMailAndNewsAgent(
				server,
				port,
				agentName,
				user,
				password,
				55,
				eTimerPeriodicHours, 
				actionEmail,
				notificationEmail,
				message,
				0,
				start, /* time in UTC */
				end, /* time in UTC */
				id   /* return value */);
	

	else if (option==2)
		rc = CreateWeeklyMailAndNewsAgent(
			server,
			port,
			agentName,
			user,
			password,
			daysvector,
			actionEmail,
			notificationEmail,
			message,
			0,
			start, /* time in UTC */
			end, /* time in UTC */
			id   /* return value */);

	 else if (option==3)
		CreateSpecificTimeMailAndNewsAgent(
            server,
            port,
            agentName,
            user,
            password,
            actionEmail,
            notificationEmail,
            message,
            0,
            start, /* time in UTC */
            id   /* return value */);

	 else if (option==4)
        rc = CreateWeeklySearchAndMailAgent(
            server,
            port,
            agentName,
            user,
            password,
            daysvector,
            actionEmail,
            notificationEmail,
            message,
            "mcom.test",
            start, /* time in UTC */
            end, /* time in UTC */
			"webpub",
			"any thing goes",
			eSearchUpdated,
            id   /* return value */);

	 else if (option==5)
        rc = CreateGETAndMailAgent(server,    /* server */
            port,               /* port */
            agentName,
            user,               /* user name */
            password,           /* passwd */
            document,           /* const char* doc, */
            event,              /* event */
            actionEmail,        /* action email */
            notificationEmail,  /* notify email */
            message,          /* message */
            0,
			getThisUrl,
            id   /* out value */
        );

	 else if (option==6) {
        rc = CreatePOSTMailAgent(server,    /* server */
            port,               /* port */
            agentName,
            user,               /* user name */
            password,           /* passwd */
            document,           /* const char* doc, */
            NS_GET_PROPERTIES_MSG,              /* event */
            actionEmail,        /* action email */
            notificationEmail,  /* notify email */
            message,          /* message */
            0,
			postToUrl,
			TRICKY_POST_MSG,
            id   /* out value */
        );
	}
    else if (option == 7)
        rc = CreateDocumentMailAndNewsAgent(server,    /* server */
            port,               /* port */
            agentName,
            user,               /* user name */
            password,           /* passwd */
            document,          /* const char* doc, */
            NS_GET_PROPERTIES_MSG,              /* event */
            actionEmail,        /* action email */
            notificationEmail,  /* notify email */
            message,          /* message */
            0,
            id   /* out value */
        );

	if ((rc == SUCCESS) && id[0])
		printf("Created Agent with id: %s\n", id);

}



